@@ -22,6 +22,11 @@
 #include <algorithm>
 #include <iostream>
 
+#include <libintl.h>
+#define _(String) gettext(String)
+#define gettext_noop(String) String
+#define N_(String) gettext_noop(String)
+
 using namespace Dbp;
 
 const int
@@ -295,7 +300,7 @@ InputGui::build() {
   // create a multiple file selection widget
   //
   GtkWindow* parentWindow = 0;
-  _selector = gtk_file_chooser_dialog_new("Add Image Files",
+  _selector = gtk_file_chooser_dialog_new(_("Add Image Files"),
     parentWindow, GTK_FILE_CHOOSER_ACTION_OPEN,
     GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
     GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT,
@@ -316,7 +321,7 @@ InputGui::build() {
   GtkCellRenderer* renderer =
     gtk_cell_renderer_text_new();
   GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes(
-    "Images", renderer,
+    _("Images"), renderer,
     "text", 0,
     (char*)0); // to fix gcc compiler warning - may be an error on 64 bit processors
   gtk_tree_view_append_column(GTK_TREE_VIEW(_inputListView), column);
@@ -329,16 +334,16 @@ InputGui::build() {
   hbox = gtk_hbox_new(TRUE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-  button = stdButton("Add Files");
+  button = stdButton(_("Add Files"));
   connect(gtkSignal(button, "clicked", &InputGui::openFileSelector, this));
   gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, FALSE, 0);
 
-  _removeButton = stdButton("Remove Files");
+  _removeButton = stdButton(_("Remove Files"));
   connect(gtkSignal(_removeButton, "clicked", &InputGui::removeFiles, this));
   gtk_box_pack_start(GTK_BOX(hbox), _removeButton, TRUE, FALSE, 0);
   gtk_widget_set_sensitive(_removeButton, FALSE);
 
-  _clearButton = stdButton("Clear List");
+  _clearButton = stdButton(_("Clear List"));
   connect(gtkSignal(_clearButton, "clicked", &InputGui::clearFileList, this));
   gtk_box_pack_start(GTK_BOX(hbox), _clearButton, TRUE, FALSE, 0);
   gtk_widget_set_sensitive(_clearButton, FALSE);
@@ -457,7 +462,7 @@ GtkWidget*
 TurnGui::build() {
   GtkWidget* vbox = gtk_vbox_new(FALSE, 0);
 
-  _enable = gtk_check_button_new_with_label("Enable");
+  _enable = gtk_check_button_new_with_label(_("Enable"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_enable), _op._enabled);
   connect(gtkSignal(_enable, "toggled", &TurnGui::setEnabled, this));
   gtk_box_pack_start(GTK_BOX(vbox), _enable, FALSE, FALSE, 0);
@@ -467,14 +472,14 @@ TurnGui::build() {
   /* pop up menu */
   GtkWidget* _turnOptions = gtk_menu_new();
 
-  addItem(_turnOptions, "Clockwise", &_turnOptionTags[0], _op._turn);
-  addItem(_turnOptions, "Upside-down", &_turnOptionTags[1], _op._turn);
-  addItem(_turnOptions, "Anti-clockwise", &_turnOptionTags[2], _op._turn);
+  addItem(_turnOptions, _("Clockwise"), &_turnOptionTags[0], _op._turn);
+  addItem(_turnOptions, _("Upside-down"), &_turnOptionTags[1], _op._turn);
+  addItem(_turnOptions, _("Anti-clockwise"), &_turnOptionTags[2], _op._turn);
 
   gtk_widget_show_all(_turnOptions);
   gtk_widget_hide(_turnOptions);
 
-  add(_controls, "Turn", _op._turn, _turnOptions);
+  add(_controls, _("Turn"), _op._turn, _turnOptions);
   if (!_op._enabled) {
     gtk_widget_set_sensitive(_controls.widget(), FALSE);
   }
@@ -502,13 +507,13 @@ GtkWidget*
 BlurGui::build() {
   GtkWidget* vbox = gtk_vbox_new(FALSE, 0);
 
-  _enable = gtk_check_button_new_with_label("Enable");
+  _enable = gtk_check_button_new_with_label(_("Enable"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_enable), _op._enabled);
   connect(gtkSignal(_enable, "toggled", &BlurGui::setEnabled, this));
   gtk_box_pack_start(GTK_BOX(vbox), _enable, FALSE, FALSE, 0);
 
   gtk_box_pack_start(GTK_BOX(vbox), _controls.widget(), FALSE, FALSE, 0);
-  add(_controls, "Radius", _op._radius, 1.0, 100.0, 1);
+  add(_controls, _("Radius"), _op._radius, 1.0, 100.0, 1);
   if (!_op._enabled) {
     gtk_widget_set_sensitive(_controls.widget(), FALSE);
   }
@@ -536,7 +541,7 @@ GtkWidget*
 RecolourGui::build() {
   GtkWidget* vbox = gtk_vbox_new(FALSE, 0);
 
-  _enable = gtk_check_button_new_with_label("Enable");
+  _enable = gtk_check_button_new_with_label(_("Enable"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_enable), _op._enabled);
   connect(gtkSignal(_enable, "toggled", &RecolourGui::setEnabled, this));
   gtk_box_pack_start(GTK_BOX(vbox), _enable, FALSE, FALSE, 0);
@@ -544,25 +549,25 @@ RecolourGui::build() {
   _controls = gtk_vbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), _controls, FALSE, FALSE, 0);
 
-  _auto = gtk_check_button_new_with_label("Auto Levels");
+  _auto = gtk_check_button_new_with_label(_("Auto Levels"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_auto), _op._auto);
   connect(gtkSignal(_auto, "toggled", &RecolourGui::setAuto, this));
   gtk_box_pack_start(GTK_BOX(_controls), _auto, FALSE, FALSE, 0);
 
-  _manualFrame = gtk_frame_new("Manual");
+  _manualFrame = gtk_frame_new(_("Manual"));
   gtk_widget_set_sensitive(_manualFrame, _op._auto ? FALSE : TRUE);
   gtk_box_pack_start(GTK_BOX(_controls), _manualFrame, FALSE, FALSE, 0);
 
   gtk_container_add(GTK_CONTAINER(_manualFrame), _manualControls.widget());
-  add(_manualControls, "Bright", _op._brightness, -1.0, 1.0, 3);
-  add(_manualControls, "Contrast", _op._contrast, -1.0, 1.0, 3);
-  add(_manualControls, "Saturation", _op._saturation, 0.0, 2.0, 3);
+  add(_manualControls, _("Bright"), _op._brightness, -1.0, 1.0, 3);
+  add(_manualControls, _("Contrast"), _op._contrast, -1.0, 1.0, 3);
+  add(_manualControls, _("Saturation"), _op._saturation, 0.0, 2.0, 3);
 #if NYI
-  add("Gamma", _op._gamma, 0.1, 10.0, 2);
+  add(_("Gamma"), _op._gamma, 0.1, 10.0, 2);
 #endif
 
   _mono = checkButtonFor(_op._mono);
-  gtk_button_set_label(GTK_BUTTON(_mono), "Convert to Grey");
+  gtk_button_set_label(GTK_BUTTON(_mono), _("Convert to Grey"));
   gtk_box_pack_start(GTK_BOX(_controls), _mono, FALSE, FALSE, 0);
 
   if (!_op._enabled) {
@@ -614,10 +619,10 @@ ResizeGui::build() {
   /* pop up menu */
   _fitOptions = gtk_menu_new();
 
-  addFitOption("Exactly", &_fitOptionTags[0]);
-  addFitOption("Padded", &_fitOptionTags[1]);
-  addFitOption("Inside", &_fitOptionTags[2]);
-  addFitOption("Outside", &_fitOptionTags[3]);
+  addFitOption(_("Exactly"), &_fitOptionTags[0]);
+  addFitOption(_("Padded"), &_fitOptionTags[1]);
+  addFitOption(_("Inside"), &_fitOptionTags[2]);
+  addFitOption(_("Outside"), &_fitOptionTags[3]);
 
   gtk_widget_show_all(_fitOptions);
   gtk_widget_hide(_fitOptions);
@@ -625,7 +630,7 @@ ResizeGui::build() {
   GtkWidget* vbox = gtk_vbox_new(FALSE, 0);
   gtk_widget_show(vbox);
 
-  _enable = gtk_check_button_new_with_label("Enable");
+  _enable = gtk_check_button_new_with_label(_("Enable"));
   connect(gtkSignal(_enable, "toggled", &ResizeGui::setEnabled, this));
   gtk_box_pack_start(GTK_BOX(vbox), _enable, FALSE, FALSE, 0);
 
@@ -633,39 +638,39 @@ ResizeGui::build() {
   gtk_box_pack_start(GTK_BOX(vbox), _controls, FALSE, FALSE, 0);
 
   GSList* relativeGroup = NULL;
-  _relativeToggle = gtk_radio_button_new_with_label(NULL, "Relative");
+  _relativeToggle = gtk_radio_button_new_with_label(NULL, _("Relative"));
   gtk_toggle_button_set_active(
     GTK_TOGGLE_BUTTON(_relativeToggle), _op._relative);
   connect(gtkSignal(_relativeToggle, "toggled", &ResizeGui::selectMode, this));
   gtk_box_pack_start(GTK_BOX(_controls), _relativeToggle, FALSE, FALSE, 0);
 
   relativeGroup = gtk_radio_button_get_group(GTK_RADIO_BUTTON(_relativeToggle));
-  _absoluteToggle = gtk_radio_button_new_with_label(relativeGroup, "Absolute");
+  _absoluteToggle = gtk_radio_button_new_with_label(relativeGroup, _("Absolute"));
   gtk_toggle_button_set_active(
     GTK_TOGGLE_BUTTON(_absoluteToggle), !_op._relative);
   gtk_box_pack_start(GTK_BOX(_controls), _absoluteToggle, FALSE, FALSE, 0);
 
-  _relativeFrame = gtk_frame_new("Relative");
+  _relativeFrame = gtk_frame_new(_("Relative"));
   gtk_widget_set_sensitive(_relativeFrame, _op._relative);
   gtk_box_pack_start(GTK_BOX(vbox), _relativeFrame, FALSE, FALSE, 0);
 
   gtk_container_add(GTK_CONTAINER(_relativeFrame), _relControls.widget());
 
-  _fixedToggle = add(_relControls, "Keep Aspect", _op._keepAspect);
+  _fixedToggle = add(_relControls, _("Keep Aspect"), _op._keepAspect);
   connect(gtkSignal(_fixedToggle, "toggled", &ResizeGui::selectAspect, this));
-  _xAdjust = add(_relControls, "X Scale", _op._xScale, 0.01, 2.0, 2);
+  _xAdjust = add(_relControls, _("X Scale"), _op._xScale, 0.01, 2.0, 2);
   connect(gtkSignal(_xAdjust, "value_changed", &ResizeGui::handleXAdjust, this));
-  _yAdjust = add(_relControls, "Y Scale", _op._yScale, 0.01, 2.0, 2);
+  _yAdjust = add(_relControls, _("Y Scale"), _op._yScale, 0.01, 2.0, 2);
   connect(gtkSignal(_yAdjust, "value_changed", &ResizeGui::handleYAdjust, this));
 
-  _absoluteFrame = gtk_frame_new("Absolute");
+  _absoluteFrame = gtk_frame_new(_("Absolute"));
   gtk_widget_set_sensitive(_absoluteFrame, ! _op._relative);
   gtk_box_pack_start(GTK_BOX(vbox), _absoluteFrame, FALSE, FALSE, 0);
 
   gtk_container_add(GTK_CONTAINER(_absoluteFrame), _absControls.widget());
 
-  add(_absControls, "Width", _op._xSize, 1, 6144);
-  add(_absControls, "Height", _op._ySize, 1, 4096);
+  add(_absControls, _("Width"), _op._xSize, 1, 6144);
+  add(_absControls, _("Height"), _op._ySize, 1, 4096);
 
   _selectFit = gtk_option_menu_new();
   gtk_option_menu_set_menu(GTK_OPTION_MENU(_selectFit), _fitOptions);
@@ -673,7 +678,7 @@ ResizeGui::build() {
 
   _warning = gtk_label_new("");
   setWarning();
-  _absControls.add("Fit", _selectFit, _warning);
+  _absControls.add(_("Fit"), _selectFit, _warning);
 
   if (!_op._enabled) {
     gtk_widget_set_sensitive(_controls, FALSE);
@@ -765,10 +770,10 @@ void
 ResizeGui::setWarning() {
   switch (_op._fit) {
   case ResizeOp::FIT_EXACTLY:
-    gtk_label_set_text(GTK_LABEL(_warning), "Note: may change aspect ratio.");
+    gtk_label_set_text(GTK_LABEL(_warning), _("Note: may change aspect ratio."));
     break;
   case ResizeOp::PAD_TO_FIT:
-    gtk_label_set_text(GTK_LABEL(_warning), "Pads with Gimp background colour");
+    gtk_label_set_text(GTK_LABEL(_warning), _("Pads with Gimp background colour"));
     break;
   case ResizeOp::FIT_INSIDE:
     gtk_label_set_text(GTK_LABEL(_warning), "");
@@ -786,17 +791,17 @@ GtkWidget*
 CropGui::build() {
   GtkWidget* vbox = gtk_vbox_new(FALSE, 0);
 
-  _enable = gtk_check_button_new_with_label("Enable");
+  _enable = gtk_check_button_new_with_label(_("Enable"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_enable), _op._enabled);
   connect(gtkSignal(_enable, "toggled", &CropGui::setEnabled, this));
   gtk_box_pack_start(GTK_BOX(vbox), _enable, FALSE, FALSE, 0);
 
   gtk_box_pack_start(GTK_BOX(vbox), _controls.widget(), FALSE, FALSE, 0);
 
-  add(_controls, "Width", _op._width, 1, 6144);
-  add(_controls, "Height", _op._height, 1, 4096);
-  add(_controls, "X Pos", _op._x, 0, 6143);
-  add(_controls, "Y Pos", _op._y, 0, 4095);
+  add(_controls, _("Width"), _op._width, 1, 6144);
+  add(_controls, _("Height"), _op._height, 1, 4096);
+  add(_controls, _("X Pos"), _op._x, 0, 6143);
+  add(_controls, _("Y Pos"), _op._y, 0, 4095);
   if (!_op._enabled) {
     gtk_widget_set_sensitive(_controls.widget(), FALSE);
   }
@@ -824,16 +829,16 @@ GtkWidget*
 SharpenGui::build() {
   GtkWidget* vbox = gtk_vbox_new(FALSE, 0);
 
-  _enable = gtk_check_button_new_with_label("Enable");
+  _enable = gtk_check_button_new_with_label(_("Enable"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_enable), _op._enabled);
   connect(gtkSignal(_enable, "toggled", &SharpenGui::setEnabled, this));
   gtk_box_pack_start(GTK_BOX(vbox), _enable, FALSE, FALSE, 0);
 
   gtk_box_pack_start(GTK_BOX(vbox), _controls.widget(), FALSE, FALSE, 0);
 
-  add(_controls, "Radius", _op._radius, 0.1, 120.0, 1);
-  add(_controls, "Amount", _op._amount, 0.0, 5.0, 2);
-  add(_controls, "Threshhold", _op._threshhold, 0.0, 255.0, 0);
+  add(_controls, _("Radius"), _op._radius, 0.1, 120.0, 1);
+  add(_controls, _("Amount"), _op._amount, 0.0, 5.0, 2);
+  add(_controls, _("Threshhold"), _op._threshhold, 0.0, 255.0, 0);
   if (!_op._enabled) {
     gtk_widget_set_sensitive(_controls.widget(), FALSE);
   }
@@ -871,23 +876,11 @@ GtkWidget*
 RenameGui::build() {
 
   // build a directory selector
-
-  _dirSelector = gtk_file_selection_new("Destination Directory");
-  gtk_window_set_modal(GTK_WINDOW(_dirSelector), TRUE);
-
-  GtkFileSelection* fs = GTK_FILE_SELECTION(_dirSelector);
-  /* Turn a file selector into a directory selector... */
-  GtkWidget* files = fs->file_list;
-  if (files->parent) {
-    files = files->parent;
-  }
-  gtk_widget_hide(files);
-  gtk_widget_hide(fs->selection_entry);
-  gtk_file_selection_hide_fileop_buttons(fs);
-
-  connect(gtkSignal(fs->ok_button, "clicked", &RenameGui::okDirSelector, this));
-  connect(gtkSignal(fs->cancel_button, "clicked", &RenameGui::cancelDirSelector, this));
-
+  _dirSelector = gtk_file_chooser_dialog_new(_("Destination Directory"),
+  	0, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+	GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+	NULL);
   /* now the page */
 
   GtkWidget* vbox = gtk_vbox_new(FALSE, 0);
@@ -895,17 +888,17 @@ RenameGui::build() {
   gtk_box_pack_start(GTK_BOX(vbox), _controls.widget(), FALSE, FALSE, 0);
 
   _dirLabel = gtk_label_new("");
-  _controls.add("To Directory:", 0, _dirLabel);
+  _controls.add(_("To Directory:"), 0, _dirLabel);
 
   GtkWidget* hbox = gtk_hbox_new(TRUE, 0);
 
   GtkWidget* button;
 
-  button = gtk_button_new_with_label("Select Dir");
+  button = gtk_button_new_with_label(_("Select Dir"));
   connect(gtkSignal(button, "clicked", &RenameGui::raiseDirSelector, this));
   gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 
-  button = gtk_button_new_with_label("Source Dir");
+  button = gtk_button_new_with_label(_("Source Dir"));
   connect(gtkSignal(button, "clicked", &RenameGui::clearDirPath, this));
   gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 
@@ -914,37 +907,37 @@ RenameGui::build() {
 
   _prefix = gtk_entry_new();
   connect(gtkSignal(_prefix, "changed", &RenameGui::setPrefix, this));
-  _controls.add("Add Prefix:", 0, _prefix);
+  _controls.add(_("Add Prefix:"), 0, _prefix);
 
   _postfix = gtk_entry_new();
   connect(gtkSignal(_postfix, "changed", &RenameGui::setPostfix, this));
-  _controls.add("Add Postfix:", 0, _postfix);
+  _controls.add(_("Add Postfix:"), 0, _postfix);
 
-  _controls.add("Example:", 0, 0);
+  _controls.add(_("Example:"), 0, 0);
   GtkWidget* label = gtk_label_new(_exampleSrc.fullPath().c_str());
-  _controls.add("Original:", 0, label);
+  _controls.add(_("Original:"), 0, label);
 
   _example = gtk_label_new("");
-  _controls.add("becomes:", 0, _example);
+  _controls.add(_("becomes:"), 0, _example);
 
-  _controls.add("Before writing:", 0, 0);
-  add(_controls, "Flatten", _op._flatten);
-  add(_controls, "Convert Grey", _op._convertToGreyscale);
-  add(_controls, "Convert Indexed", _op._convertToIndexed);
+  _controls.add(_("Before writing:"), 0, 0);
+  add(_controls, _("Flatten"), _op._flatten);
+  add(_controls, _("Convert Grey"), _op._convertToGreyscale);
+  add(_controls, _("Convert Indexed"), _op._convertToIndexed);
 
   /* pop up menu */
   GtkWidget* ditherOptions = gtk_menu_new();
 
-  addItem(ditherOptions, "No Dither", &_ditherOptionTags[0], _op._ditherType);
-  addItem(ditherOptions, "Floyd-Steinberg", &_ditherOptionTags[1], _op._ditherType);
-  addItem(ditherOptions, "Reduced Bleed", &_ditherOptionTags[2], _op._ditherType);
-  addItem(ditherOptions, "Fixed Position", &_ditherOptionTags[3], _op._ditherType);
+  addItem(ditherOptions, _("No Dither"), &_ditherOptionTags[0], _op._ditherType);
+  addItem(ditherOptions, _("Floyd-Steinberg"), &_ditherOptionTags[1], _op._ditherType);
+  addItem(ditherOptions, _("Reduced Bleed"), &_ditherOptionTags[2], _op._ditherType);
+  addItem(ditherOptions, _("Fixed Position"), &_ditherOptionTags[3], _op._ditherType);
 
   gtk_widget_show_all(ditherOptions);
   gtk_widget_hide(ditherOptions);
 
-  add(_controls, "Dither", _op._ditherType, ditherOptions);
-  add(_controls, "Colours", _op._numberOfIndexedColours, 2, 256);
+  add(_controls, _("Dither"), _op._ditherType, ditherOptions);
+  add(_controls, _("Colours"), _op._numberOfIndexedColours, 2, 256);
 
   recalcExample();
   recalcDirName();
@@ -956,7 +949,7 @@ RenameGui::build() {
 void
 RenameGui::recalcDirName() {
   if (_op._dirPath.empty()) {
-    gtk_label_set_text(GTK_LABEL(_dirLabel), "same as source");
+    gtk_label_set_text(GTK_LABEL(_dirLabel), _("same as source"));
   } else {
     gtk_label_set_text(GTK_LABEL(_dirLabel), _op._dirPath.c_str());
   }
@@ -993,11 +986,12 @@ RenameGui::clearDirPath() {
 
 void
 RenameGui::okDirSelector() {
-  std::string dir = 
-    gtk_file_selection_get_filename(GTK_FILE_SELECTION(_dirSelector));
+  char* filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(_dirSelector));
+  std::string dir = filename;
+  g_free(filename);
   // file selector returns slash-terminated directory path, remove slash
   int len = dir.length();
-  if ((len > 0) && (dir[len - 1] == '/')) {
+  if ((len > 0) && (dir[len - 1] == G_DIR_SEPARATOR)) {
     dir.erase(len - 1);
   }
 
@@ -1014,7 +1008,11 @@ RenameGui::cancelDirSelector() {
 
 void
 RenameGui::raiseDirSelector() {
-  gtk_widget_show(_dirSelector);
+  if (gtk_dialog_run(GTK_DIALOG(_dirSelector)) == GTK_RESPONSE_ACCEPT) {
+  	okDirSelector();
+  } else {
+    cancelDirSelector();
+  }
 }
 
 OutputGui::OutputGui(OutputOp& op):
@@ -1040,7 +1038,7 @@ OutputGui::build() {
   GtkWidget* hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-  GtkWidget* label = gtk_label_new("Format:");
+  GtkWidget* label = gtk_label_new(_("Format:"));
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 4);
 
   GtkWidget* formats = gtk_option_menu_new();
@@ -1146,7 +1144,7 @@ DbpGui::build() {
   gimp_ui_init(_name.c_str(), usesPreview);
   GtkDialogFlags flags = GtkDialogFlags(0);
   dialog = gimp_dialog_new(
-    "David's Batch Processor",
+    _("David's Batch Processor"),
     _name.c_str(),
     0, flags, 0, 0,
     GTK_STOCK_QUIT, 0,
@@ -1162,7 +1160,7 @@ DbpGui::build() {
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), _notebook, TRUE, TRUE, 0);
   // insert pages later...
 
-  GtkWidget* frame = gtk_frame_new("Processing");
+  GtkWidget* frame = gtk_frame_new(_("Processing"));
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, FALSE, FALSE, 5);
 
   GtkWidget* vbox = gtk_vbox_new(FALSE, 0);
@@ -1178,20 +1176,20 @@ DbpGui::build() {
   GtkWidget* hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-  _start = Gui::stdButton("Start");
+  _start = Gui::stdButton(_("Start"));
   connect(gtkSignal(_start, "clicked", &DbpGui::start, this));
   gtk_box_pack_start(GTK_BOX(hbox), _start, FALSE, FALSE, 0);
 
-  _cancel = Gui::stdButton("Cancel");
+  _cancel = Gui::stdButton(_("Cancel"));
   connect(gtkSignal(_cancel, "clicked", &DbpGui::cancel, this));
   gtk_box_pack_start(GTK_BOX(hbox), _cancel, FALSE, FALSE, 0);
 
-  _test = Gui::stdButton("Test");
+  _test = Gui::stdButton(_("Test"));
   connect(gtkSignal(_test, "clicked", &DbpGui::test, this));
   gtk_box_pack_start(GTK_BOX(hbox), _test, FALSE, FALSE, 0);
 
   _data.setVisible(false);
-  _show = Gui::stdToggle("Show Images");
+  _show = Gui::stdToggle(_("Show Images"));
   connect(gtkSignal(_show, "toggled", &DbpGui::show, this));
   gtk_box_pack_end(GTK_BOX(hbox), _show, FALSE, FALSE, 0);
 
@@ -1201,15 +1199,15 @@ DbpGui::build() {
 
   // now add other widgets, so they don't get show_all()
   //
-  add("Input", new InputGui(_data._input));
-  add("Turn", new TurnGui(_data._turn));
-  add("Blur", new BlurGui(_data._blur));
-  add("Colour", new RecolourGui(_data._recolour));
-  add("Resize", new ResizeGui(_data._resize));
-  add("Crop", new CropGui(_data._crop));
-  add("Sharpen", new SharpenGui(_data._sharpen));
-  add("Rename", new RenameGui(_data._rename));
-  add("Output", new OutputGui(_data._output));
+  add(_("Input"), new InputGui(_data._input));
+  add(_("Turn"), new TurnGui(_data._turn));
+  add(_("Blur"), new BlurGui(_data._blur));
+  add(_("Colour"), new RecolourGui(_data._recolour));
+  add(_("Resize"), new ResizeGui(_data._resize));
+  add(_("Crop"), new CropGui(_data._crop));
+  add(_("Sharpen"), new SharpenGui(_data._sharpen));
+  add(_("Rename"), new RenameGui(_data._rename));
+  add(_("Output"), new OutputGui(_data._output));
 
   return dialog;
 }
@@ -1245,7 +1243,7 @@ DbpGui::test() {
 
   // need a test file...
   if (_data._input._files.empty()) {
-    gtk_label_set_text(GTK_LABEL(_messageText), "No file to test with!");
+    gtk_label_set_text(GTK_LABEL(_messageText), _("No file to test with!"));
     return;
   }
 
@@ -1260,7 +1258,7 @@ DbpGui::start() {
 
   int n = _data.numExistingOutputFiles(_data._input._files);
   if (n > 0) {
-    gtk_label_set_text(GTK_LABEL(_messageText), "Output files exist, please delete them!");
+    gtk_label_set_text(GTK_LABEL(_messageText), _("Output files exist, please delete them!"));
     return;
   }
 
@@ -1272,7 +1270,7 @@ DbpGui::start() {
 bool
 DbpGui::step() {
   if (_data.done()) {
-    gtk_label_set_text(GTK_LABEL(_messageText), "-- done --");
+    gtk_label_set_text(GTK_LABEL(_messageText), _("-- done --"));
     setBusy(false);
     return false;
   }
