@@ -26,8 +26,6 @@
 
 #include "libgimp/stdplugins-intl.h"
 
-#include "focusblur.h"
-
 #include "fftblur.h"
 #include "fftblurbuffer.h"
 #include "fftblurproc.h"
@@ -65,15 +63,22 @@ struct _ConvertShineSet
 
 /*---- Functions for iterator ----*/
 
+inline static gfloat
+convf_shine_sub (ConvertShineSet *ss,
+                 gint             x,
+                 gint             y)
+{
+  gint s1 = focusblur_shine_get (ss->shine, x, y);
+  return focusblur_diffusion_get_shine (ss->diffusion, ss->level, s1);
+}
+
 static void
 convf_shine0 (gfloat    *workp,
               gint       x,
               gint       y,
               gpointer   data)
 {
-  ConvertShineSet *ss = (ConvertShineSet *) data;
-  gint sl = focusblur_shine_get (ss->shine, x, y);
-  *workp = focusblur_diffusion_get_shine (ss->diffusion, ss->level, sl);
+  *workp = convf_shine_sub ((ConvertShineSet *) data, x, y);
 }
 
 static void
@@ -82,9 +87,7 @@ convf_shinex (gfloat    *workp,
               gint       y,
               gpointer   data)
 {
-  ConvertShineSet *ss = (ConvertShineSet *) data;
-  gint sl = focusblur_shine_get (ss->shine, x, y);
-  *workp *= focusblur_diffusion_get_shine (ss->diffusion, ss->level, sl);
+  *workp *= convf_shine_sub ((ConvertShineSet *) data, x, y);
 }
 
 static void
@@ -624,7 +627,7 @@ focusblur_fft_render_depth_precedence (FblurParam       *param,
 
   if (progress_bar)
     full_progress = 2 + fft->depth.count;
-  if (param->store.enable_depth_fill_behind)
+  /* fill behind */
     full_progress += 1;
 
   transit = focusblur_fft_work_add_buffer (fft);
@@ -746,62 +749,61 @@ focusblur_fft_render_depth_precedence (FblurParam       *param,
       has_alpha)
     focusblur_fft_work_copy (fft, sum[channels], sum[6]);
 
-  if (param->store.enable_depth_fill_behind)
+  /* fill behind */
+  level = FBLUR_DEPTH_MAX;
+  focusblur_fft_buffer_make_kernel (fft, param->diffusion, level);
+
+  /* this is imperfect */
+  focusblur_fft_buffer_make_depth_behind (fft, param->depth_map);
+
+  focusblur_fft_work_treat_outside (fft);
+  focusblur_fft_work_store (fft, slice0);
+
+  focusblur_fft_work_apply (fft);
+  focusblur_fft_work_store (fft, slice1);
+
+  if (has_alpha)
+    /* need to more tests */
+    focusblur_fft_convert_work (fft, convf_fill0, sum[channels]);
+  else
+    focusblur_fft_convert_work (fft, convf_fill0, sum[bpp]);
+
+  focusblur_fft_work_store (fft, transit);
+
+  focusblur_fft_work_restore (fft, slice1);
+  focusblur_fft_convert_work (fft, convf_mul, transit);
+  focusblur_fft_convert_work (fft, convf_radd, sum[bpp]);
+
+  if (enable_shine)
     {
-      level = FBLUR_DEPTH_MAX;
-      focusblur_fft_buffer_make_kernel (fft, param->diffusion, level);
-
-      /* this is imperfect */
-      focusblur_fft_buffer_make_depth_behind (fft, param->depth_map);
+      focusblur_fft_convert_work (fft, convf_radd, sum[5]);
+    }
 
+  if (has_alpha)
+    {
+      focusblur_fft_work_restore (fft, slice0);
+      focusblur_fft_convert_source2work (fft, convf_alphax, channels);
       focusblur_fft_work_treat_outside (fft);
       focusblur_fft_work_store (fft, slice0);
-
       focusblur_fft_work_apply (fft);
-      focusblur_fft_work_store (fft, slice1);
-
-      if (has_alpha)
-        /* need to more tests */
-        focusblur_fft_convert_work (fft, convf_fill0, sum[channels]);
-      else
-        focusblur_fft_convert_work (fft, convf_fill0, sum[bpp]);
-
-      focusblur_fft_work_store (fft, transit);
-
-      focusblur_fft_work_restore (fft, slice1);
       focusblur_fft_convert_work (fft, convf_mul, transit);
-      focusblur_fft_convert_work (fft, convf_radd, sum[bpp]);
-
-      if (enable_shine)
-        {
-          focusblur_fft_convert_work (fft, convf_radd, sum[5]);
-        }
-
-      if (has_alpha)
-        {
-          focusblur_fft_work_restore (fft, slice0);
-          focusblur_fft_convert_source2work (fft, convf_alphax, channels);
-          focusblur_fft_work_treat_outside (fft);
-          focusblur_fft_work_store (fft, slice0);
-          focusblur_fft_work_apply (fft);
-          focusblur_fft_convert_work (fft, convf_mul, transit);
-          focusblur_fft_convert_work (fft, convf_radd, sum[channels]);
-        }
-
-      for (c = 0; c < channels; c ++)
-        {
-          focusblur_fft_work_restore (fft, slice0);
-          focusblur_fft_convert_source2work (fft, convf_colorx, c);
-          focusblur_fft_work_treat_outside (fft);
-          focusblur_fft_work_apply (fft);
-          focusblur_fft_convert_work (fft, convf_mul, transit);
-          focusblur_fft_convert_work (fft, convf_radd, sum[c]);
-        }
-
-      if (progress_bar)
-        gimp_progress_update ((gdouble) ++ progress / full_progress);
+      focusblur_fft_convert_work (fft, convf_radd, sum[channels]);
     }
 
+  for (c = 0; c < channels; c ++)
+    {
+      focusblur_fft_work_restore (fft, slice0);
+      focusblur_fft_convert_source2work (fft, convf_colorx, c);
+      focusblur_fft_work_treat_outside (fft);
+      focusblur_fft_work_apply (fft);
+      focusblur_fft_convert_work (fft, convf_mul, transit);
+      focusblur_fft_convert_work (fft, convf_radd, sum[c]);
+    }
+
+  /* end of fill behind */
+  if (progress_bar)
+    gimp_progress_update ((gdouble) ++ progress / full_progress);
+
   if (enable_shine)
     focusblur_fft_convert_source (fft, convf_fill5, sum);
 
