@@ -1,5 +1,5 @@
 /* Focus Blur -- blur with focus plug-in.
- * Copyright (C) 2002-2007 Kyoichiro Suda
+ * Copyright (C) 2002-2008 Kyoichiro Suda
  *
  * The GIMP -- an image manipulation program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
@@ -27,21 +27,18 @@
 #  include <math.h>
 #endif
 #include <fftw3.h>
-
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
 
-#include "focusblur.h"
+#include "libgimp/stdplugins-intl.h"
 
 #include "focusblurenums.h"
 #include "focusblurtypes.h"
 #include "focusblurparam.h"
-
+#include "gimpplugin.h"
 #include "diffusion.h"
 #include "depthmap.h"
 #include "shine.h"
-#include "interface.h"
-
 #include "fftblurbuffer.h"
 #include "fftblurproc.h"
 
@@ -80,9 +77,9 @@ focusblur_fft_buffer_update (FblurFftBuffer **fft,
                              GimpPreview     *preview)
 {
   FblurQualityType       quality;
-  gint                   x, y;
   gint                   width, height;
   gint                   radius, range;
+  gint                   x1, x2, y1, y2;
 
   quality = preview ? param->pref.quality_preview : param->pref.quality;
 
@@ -91,7 +88,15 @@ focusblur_fft_buffer_update (FblurFftBuffer **fft,
   if (quality == FBLUR_QUALITY_BEST)
     return FALSE;
 
-  focusblur_diffusion_update (&(param->diffusion), *fft, &(param->store));
+  if (! focusblur_diffusion_update (&(param->diffusion), *fft,
+                                    &(param->store), param->brush_name))
+    {
+      gimp_message (_("Failed to update diffusion table."));
+
+      focusblur_fft_buffer_destroy (fft);
+      return FALSE;
+    }
+
   radius = ceilf (param->diffusion->model_radius); // without softness
   range = param->diffusion->model_radius_int;
 
@@ -101,13 +106,34 @@ focusblur_fft_buffer_update (FblurFftBuffer **fft,
        radius > 63)) // needs more tune
     return FALSE;
 
-  if (! preview)
-    gimp_drawable_mask_intersect (param->drawable_ID, &x, &y, &width, &height);
-  else
-    gimp_preview_get_size (GIMP_PREVIEW (preview), &width, &height); 
+  gimp_drawable_mask_bounds (param->drawable_ID, &x1, &y1, &x2, &y2);
 
-  if (width  <= range ||
-      height <= range)
+  if (preview)
+    {
+      gint px1, px2, py1, py2;
+      gint pw, ph;
+
+      gimp_preview_get_position (GIMP_PREVIEW (preview), &px1, &py1); 
+      gimp_preview_get_size (GIMP_PREVIEW (preview), &pw, &ph); 
+      px2 = px1 + pw;
+      py2 = py1 + ph;
+
+      /* it is not need for GimpDrawablePreview */
+      if (x1 >= px2 || x2 <= px1 ||
+          y1 >= py2 || y2 <= py1)
+        return TRUE;
+
+      x1 = MAX (px1, x1);
+      x2 = MIN (px2, x2);
+      y1 = MAX (py1, y1);
+      y2 = MIN (py2, y2);
+    }
+
+  width  = x2 - x1;
+  height = y2 - y1;
+
+  if (width  < range ||
+      height < range)
     return FALSE;
 
   if (! *fft)
@@ -116,8 +142,7 @@ focusblur_fft_buffer_update (FblurFftBuffer **fft,
 
       if (! *fft)
         {
-          g_printerr ("%s: %s(): failed to allocate memory.\n",
-                      PLUG_IN_BINARY, G_STRFUNC);
+          gimp_message (_("Failed to allocate memory."));
 
           return FALSE;
         }
@@ -128,8 +153,7 @@ focusblur_fft_buffer_update (FblurFftBuffer **fft,
   /* fftsource must be updated before to update work and depth */
   if (! focusblur_fft_buffer_update_source (*fft, param->drawable, preview))
     {
-      g_printerr ("%s: %s(): failed to update source buffer.\n",
-                  PLUG_IN_BINARY, G_STRFUNC);
+      gimp_message (_("Failed to update source buffer."));
 
       focusblur_fft_buffer_destroy (fft);
       return FALSE;
@@ -140,10 +164,10 @@ focusblur_fft_buffer_update (FblurFftBuffer **fft,
       if (! focusblur_depth_map_update (&(param->depth_map), *fft,
                                         &(param->store)))
         {
-          g_printerr ("%s: %s(): failed to update depth info, but continue.\n",
-                      PLUG_IN_BINARY, G_STRFUNC);
+          gimp_message (_("Failed to update depth info."));
 
-          focusblur_widget_set_enable_depth_map (param, FALSE);
+          focusblur_fft_buffer_destroy (fft);
+          return FALSE;
         }
       else
         {
@@ -155,8 +179,7 @@ focusblur_fft_buffer_update (FblurFftBuffer **fft,
 
   if (! focusblur_fft_buffer_update_work (*fft, range))
     {
-      g_printerr ("%s: %s(): failed to update working buffer.\n",
-                  PLUG_IN_BINARY, G_STRFUNC);
+      gimp_message (_("Failed to update working buffer."));
 
       focusblur_fft_buffer_destroy (fft);
       return FALSE;
@@ -165,10 +188,10 @@ focusblur_fft_buffer_update (FblurFftBuffer **fft,
   if (! focusblur_shine_update (&(param->shine), param->drawable,
                                 &(param->store)))
     {
-      g_printerr ("%s: %s(): failed to update shine data, but continue.\n",
-                  PLUG_IN_BINARY, G_STRFUNC);
+      gimp_message (_("Failed to update shine data."));
 
-      focusblur_widget_set_enable_shine (param, FALSE);
+      focusblur_fft_buffer_destroy (fft);
+      return FALSE;
     }
 
   return TRUE;
@@ -193,20 +216,38 @@ focusblur_fft_buffer_destroy (FblurFftBuffer **fft)
 void
 focusblur_fft_buffer_draw (FblurFftBuffer *fft)
 {
-  GimpPixelRgn   pr;
-  gint x, y;
+  GimpDrawablePreview *preview;
+  gboolean             dirty;
+  guint8              *data;
+  GimpPixelRgn         pr;
+  gint                 x, y;
 
   if (! fft->source.preview)
     {
-      g_assert (fft->source.data);
+      preview = NULL;
+      dirty = TRUE;
+      data = fft->source.data;
+    }
+  else
+    {
+      preview = GIMP_DRAWABLE_PREVIEW (fft->source.preview);
+      g_assert (preview != NULL);
 
-      gimp_pixel_rgn_init (&pr, fft->source.drawable,
+      dirty = FALSE;
+      data = fft->source.data_preview;
+    }
+
+  g_assert (data != NULL);
+
+  gimp_pixel_rgn_init (&pr, fft->source.drawable,
+                       fft->source.x1, fft->source.y1,
+                       fft->source.width, fft->source.height, dirty, TRUE);
+  gimp_pixel_rgn_set_rect (&pr, data,
                            fft->source.x1, fft->source.y1,
-                           fft->source.width, fft->source.height, TRUE, TRUE);
-      gimp_pixel_rgn_set_rect (&pr, fft->source.data,
-                               fft->source.x1, fft->source.y1,
-                               fft->source.width, fft->source.height);
+                           fft->source.width, fft->source.height);
 
+  if (! preview)
+    {
       gimp_drawable_flush (fft->source.drawable);
       gimp_drawable_merge_shadow (fft->source.drawable->drawable_id, TRUE);
       gimp_drawable_update (fft->source.drawable->drawable_id,
@@ -218,34 +259,8 @@ focusblur_fft_buffer_draw (FblurFftBuffer *fft)
     }
   else
     {
-      /* preview area size */
-      if (fft->source.data_preview)
-        {
-          gimp_preview_area_draw
-            (GIMP_PREVIEW_AREA (gimp_preview_get_area (fft->source.preview)),
-             0, 0, fft->source.width, fft->source.height,
-             gimp_drawable_type (fft->source.drawable->drawable_id),
-             fft->source.data_preview, fft->source.rowstride);
-        }
-      /* non-cropped image (not implemented) */
-      else
-        {
-          g_assert (fft->source.data);
+      gimp_drawable_preview_draw_region (preview, &pr);
 
-          gimp_preview_get_position
-            (GIMP_PREVIEW (fft->source.preview), &x, &y);
-
-          x = fft->source.x1 - x;
-          y = fft->source.y1 - y;
-
-          gimp_preview_area_draw
-            (GIMP_PREVIEW_AREA (gimp_preview_get_area (fft->source.preview)),
-             x, y, fft->source.width, fft->source.height,
-             gimp_drawable_type (fft->source.drawable->drawable_id),
-             fft->source.data, fft->source.rowstride);
-
-          focusblur_fft_buffer_clear_source (fft);
-        }
     }
 }
 
@@ -416,18 +431,30 @@ focusblur_fft_buffer_update_source (FblurFftBuffer *fft,
 
   fft->source.drawable = drawable;
   fft->source.preview  = preview;
-  if (! preview)
+
+  gimp_drawable_mask_bounds (drawable->drawable_id, &x1, &y1, &x2, &y2);
+
+  if (preview)
     {
-      gimp_drawable_mask_intersect
-        (drawable->drawable_id, &x1, &y1, &width, &height);
+      gint px1, px2, py1, py2;
+      gint pw, ph;
+
+      gimp_preview_get_position (GIMP_PREVIEW (preview), &px1, &py1); 
+      gimp_preview_get_size (GIMP_PREVIEW (preview), &pw, &ph); 
+      px2 = px1 + pw;
+      py2 = py1 + ph;
+
+      x1 = MAX (px1, x1);
+      x2 = MIN (px2, x2);
+      y1 = MAX (py1, y1);
+      y2 = MIN (py2, y2);
     }
-  else
-    {
-      gimp_preview_get_position (GIMP_PREVIEW (preview), &x1, &y1);
-      gimp_preview_get_size (GIMP_PREVIEW (preview), &width, &height); 
-    }
-  x2 = x1 + width;
-  y2 = y1 + height;
+
+  g_assert (x1 < x2);
+  g_assert (y1 < y2);
+
+  width  = x2 - x1;
+  height = y2 - y1;
 
   ntiles = 1 + x2 / TILE_WIDTH - x1 / TILE_WIDTH;
   gimp_tile_cache_ntiles (ntiles);
@@ -546,9 +573,11 @@ focusblur_fft_buffer_update_work (FblurFftBuffer *fft,
   fft->work.complex_nelements = fft->work.nelements / 2;
   fft->work.size = sizeof (fftwf_complex) * fft->work.complex_nelements;
 
-  fft->work.size += 15;
-  fft->work.size &= ~15;
+  /* 32-bytes pair (4x complex or 8x real) processing */
+  fft->work.size += 31;
+  fft->work.size &= ~31;
 
+  /* fftwf_malloc() (or distributed package) is broken. */
   fft->work.image  = fftwf_malloc (fft->work.size);
   fft->work.kernel = fftwf_malloc (fft->work.size);
   if (! fft->work.image || ! fft->work.kernel)
