@@ -22,23 +22,18 @@
 #ifndef __FOCUSBLUR_H__
 #define __FOCUSBLUR_H__
 
-#include <libgimp/gimp.h>
+#include <glib/gmacros.h>
 
 G_BEGIN_DECLS
 
 
-#define PLUG_IN_BINARY  "focusblur"
-#define PLUG_IN_PROC    "plug-in-focusblur"
-
-#define TILE_WIDTH  gimp_tile_width()
-#define TILE_HEIGHT gimp_tile_height()
-
-#define FBLUR_MAX_NUM_THREADS 16
-#define FBLUR_RADIUS_MAX 127
+#define FBLUR_MAX_NUM_THREADS   16
+#define FBLUR_RADIUS_MAX        127
 /* do not set over the precision of 1 byte (255),
    and it is hard-coded in [012]depthmap.c, fftblur*.c */
-#define FBLUR_DEPTH_MAX 127
-#define FBLUR_DIFFUSION_NTABLES (1 + 2 * FBLUR_DEPTH_MAX)
+#define FBLUR_DEPTH_MAX         127
+#define FBLUR_DIFFUSION_NTABLES (1 + FBLUR_DEPTH_MAX)
+#define FBLUR_GAUSSIAN_FACTOR   0.3003866304f
 
 
 G_END_DECLS
