@@ -28,39 +28,20 @@
 
 #include "focusblur.h"
 #include "focusblurparam.h"
+#include "focusblurstock.h"
+#include "gimpplugin.h"
 #include "interface.h"
 #include "render.h"
-#include "fftblur.h"
-
-#include "source.h"
-#include "diffusion.h"
-#include "depthmap.h"
-#include "shine.h"
-
-
-/*---- Types ----*/
-
-#ifdef ENABLE_MP
-typedef struct _FblurThreadSet FblurThreadSet;
-struct _FblurThreadSet
-{
-  GimpPixelRgn  *region;
-  volatile gint *waiting;
-  gint           number;
-};
-#endif
 
 
 /*---- Prototypes ----*/
 
-static void     focusblur_execute_region        (FblurParam     *param,
-                                                 GimpPixelRgn   *region,
-                                                 gint            number);
 static GtkWidget* focusblur_notebook_append_vbox  (GtkNotebook  *notebook,
                                                    gchar        *label_text);
 static GtkWidget* focusblur_notebook_append_table (GtkNotebook  *notebook,
                                                    gchar        *label_text,
                                                    gint          rows);
+
 static void     focusblur_preview_invalidate    (GimpPreview    *gimppreview,
                                                  FblurParam     *param);
 static void     focusblur_widget_pickup_focal_depth (FblurParam *param,
@@ -72,162 +53,19 @@ static void     focusblur_preview_button_press  (GtkWidget      *widget,
 static gboolean focusblur_map_constraint        (gint32          image_ID,
                                                  gint32          drawable_ID,
                                                  gpointer        data);
+static void     focusblur_change_brush_name     (GObject        *object,
+                                                 GParamSpec     *arg1,
+                                                 gpointer        user_data);
+static void     focusblur_update_enumcombobox_text (GimpEnumComboBox *combo,
+                                                    gint              index,
+                                                    const gchar      *text);
+static void     focusblur_update_widget_brushmodel (GObject     *object,
+                                                    GParamSpec  *arg1,
+                                                    gpointer     user_data);
+
 
 /*---- Funcs ----*/
 
-gboolean
-focusblur_execute (FblurParam   *param,
-                   GimpPreview  *preview)
-{
-  GimpPixelRgn   dest_rgn;
-  gpointer       reg_rgn;
-  gdouble        progress, full_progress;
-  gint           x1, y1, width, height;
-
-  /* Nothing to do */
-  if (param->store.model_radius <= 0.0f)
-    return TRUE;
-
-#ifdef HAVE_FFTW3
-  if (focusblur_fft_execute (param, preview))
-    return TRUE;
-#endif
-
-  if (! focusblur_param_prepare (param))
-    return FALSE;
-
-  if (! preview)
-    {
-      x1 = param->source->x1;
-      y1 = param->source->y1;
-      width  = param->source->x2 - param->source->x1;
-      height = param->source->y2 - param->source->y1;
-    }
-  else
-    {
-      gimp_preview_get_position (GIMP_PREVIEW (preview), &x1, &y1);
-      gimp_preview_get_size (GIMP_PREVIEW (preview), &width, &height);
-    }
-
-  gimp_tile_cache_ntiles (16);
-
-  /* Destined image */
-  gimp_pixel_rgn_init (&dest_rgn, param->drawable,
-                       x1, y1, width, height, (preview == NULL), TRUE);
-
-  progress = full_progress = 0.0;
-  if (! preview)
-    {
-      full_progress = width * height;
-      progress = 0.0;
-      gimp_progress_init (_("Focus Blur..."));
-      gimp_progress_update (0.0001);
-    }
-
-  for (reg_rgn = gimp_pixel_rgns_register (1, &dest_rgn);
-       reg_rgn != NULL; reg_rgn = gimp_pixel_rgns_process (reg_rgn))
-    {
-#ifdef ENABLE_MP
-      if (param->max_threads)
-        {
-          volatile gint  num = param->max_threads;
-          FblurThreadSet set[param->max_threads];
-          gint           i;
-
-          for (i = 0; i < param->max_threads; i ++)
-            {
-              set[i].region  = &dest_rgn;
-              set[i].waiting = &num;
-              set[i].number  =  i;
-              g_thread_pool_push (param->thread_pool, &(set[i]), NULL);
-            }
-          focusblur_execute_region (param, &dest_rgn, param->max_threads);
-          while (g_atomic_int_get (&num))
-            g_thread_yield ();
-        }
-      else
-#endif /* ENABLE_MP */
-        focusblur_execute_region (param, &dest_rgn, 0);
-
-      if (! preview)
-        {
-          progress += (dest_rgn.w * dest_rgn.h);
-          gimp_progress_update (progress / full_progress);
-        }
-      else
-        {
-          gimp_drawable_preview_draw_region (GIMP_DRAWABLE_PREVIEW (preview),
-                                             &dest_rgn);
-        }
-    }
-
-  if (! preview)
-    {
-      /* update the blurred region */
-      gimp_drawable_flush (param->drawable);
-      gimp_drawable_merge_shadow (param->drawable_ID, TRUE);
-      gimp_drawable_update (param->drawable_ID, x1, y1, width, height);
-    }
-
-  return TRUE;
-}
-
-
-#ifdef ENABLE_MP
-void
-focusblur_execute_thread (gpointer      data,
-                          gpointer      user_data)
-{
-  FblurThreadSet        *thread_set = data;
-  FblurParam            *param = user_data;
-
-  focusblur_execute_region (param, thread_set->region, thread_set->number);
-  g_atomic_int_add (thread_set->waiting, -1);
-}
-#endif /* ENABLE_MP */
-
-
-static void
-focusblur_execute_region (FblurParam    *param,
-                          GimpPixelRgn  *region,
-                          gint           number)
-{
-  guchar        *blp, *bp;
-  gint           bpp = param->drawable->bpp;
-  gint           rowstride = region->rowstride;
-  gint           x1, x2, x;
-  gint           y1, y2, y;
-
-  blp = region->data;
-
-  x1 = region->x;
-  x2 = x1 + region->w;
-
-  y1 = region->y;
-  y2 = y1 + region->h;
-
-#ifdef ENABLE_MP
-  if (param->max_threads)
-    {
-      gint h, h0, y3;
-      g_assert (number <= param->max_threads);
-
-      h = (region->h + param->max_threads) / (param->max_threads + 1);
-      h0 = number * h;
-      y1 += h0;
-      blp += h0 * rowstride;
-      y3 = y1 + h;
-      if (y3 < y2)
-        y2 = y3;
-    }
-#endif
-
-  for (y = y1; y < y2; y ++, blp += rowstride)
-    for (x = x1, bp = blp; x < x2; x ++, bp += bpp)
-      focusblur_render_pixel (x, y, bp, bpp, param);
-}
-
-
 static GtkWidget*
 focusblur_notebook_append_vbox (GtkNotebook *notebook,
                                 gchar       *label_text)
@@ -276,16 +114,19 @@ focusblur_dialog (FblurParam *param)
   GtkWidget     *vbox;
   GtkWidget     *hbox;
   GtkWidget     *label;
-  GtkWidget     *align;
+  GtkWidget     *button;
   GtkWidget     *spinbutton;
   GtkObject     *adj;
   GtkWidget     *combo_box;
   GtkWidget     *toggle;
   GtkWidget     *table;
+  GtkListStore  *enum_store;
+  gchar         *text;
   gint           row;
   gboolean       run;
 
   gimp_ui_init (PLUG_IN_BINARY, TRUE);
+  focusblur_stock_init ();
 
   /* Main dialogue */
 
@@ -317,14 +158,15 @@ focusblur_dialog (FblurParam *param)
   gtk_box_pack_start (GTK_BOX (main_vbox), preview, FALSE, TRUE, 0);
   gtk_widget_show (preview);
 
-  g_signal_connect (preview, "invalidated",
+  g_signal_connect (G_OBJECT (preview), "invalidated",
                     G_CALLBACK (focusblur_preview_invalidate),
                     param);
-  g_signal_connect (gimp_preview_get_area (GIMP_PREVIEW (preview)),
+  g_signal_connect (G_OBJECT (gimp_preview_get_area (GIMP_PREVIEW (preview))),
                     "button_press_event",
                     G_CALLBACK (focusblur_preview_button_press),
                     param);
 
+
   /* Notebook to select options */
 
   notebook = gtk_notebook_new ();
@@ -338,8 +180,8 @@ focusblur_dialog (FblurParam *param)
 
   /* Basic parameters */
 
-  nb_vbox = focusblur_notebook_append_vbox
-    (GTK_NOTEBOOK (notebook), _("_Basic"));
+  nb_vbox = focusblur_notebook_append_vbox (GTK_NOTEBOOK (notebook),
+                                            _("_Basic"));
 
   /* Diffusion Radius */
 
@@ -351,21 +193,18 @@ focusblur_dialog (FblurParam *param)
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
-  align = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
-  gtk_box_pack_start_defaults (GTK_BOX (hbox), align);
-  gtk_widget_show (align);
-
   label = gtk_label_new_with_mnemonic (_("Diffusion Model and _Radius:"));
-  gtk_container_add (GTK_CONTAINER (align), label);
+  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+  gtk_box_pack_start_defaults (GTK_BOX (hbox), label);
   gtk_widget_show (label);
 
   spinbutton = gimp_spin_button_new (&adj, param->store.model_radius,
                                      0.0, 100.0, 1.0, 10.0, 0.0, 1.0, 2);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), spinbutton);
-  g_signal_connect (adj, "value-changed",
+  g_signal_connect (G_OBJECT (adj), "value-changed",
                     G_CALLBACK (gimp_float_adjustment_update),
                     &(param->store.model_radius));
-  g_signal_connect_swapped (adj, "value-changed",
+  g_signal_connect_swapped (G_OBJECT (adj), "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
   gtk_box_pack_start (GTK_BOX (hbox), spinbutton, FALSE, FALSE, 0);
@@ -373,47 +212,49 @@ focusblur_dialog (FblurParam *param)
 
   /* Diffusion Model */
 
-  combo_box = gimp_int_combo_box_new (_("Flat"),    FBLUR_MODEL_FLAT,
-                                      _("Ring"),    FBLUR_MODEL_RING,
-                                      _("Concave"), FBLUR_MODEL_CONCAVE,
-                                      NULL);
+  combo_box = gimp_enum_combo_box_new (FBLUR_TYPE_MODEL_TYPE);
+  gimp_enum_combo_box_set_stock_prefix
+    (GIMP_ENUM_COMBO_BOX (combo_box), "focusblur-model");
+  /* change label */
+  text = g_strdup_printf (_("Brush:%s"), param->brush_name);
+  focusblur_update_enumcombobox_text (GIMP_ENUM_COMBO_BOX (combo_box),
+                                      FBLUR_MODEL_BRUSH, text);
+  g_free (text);
+
   gimp_int_combo_box_connect (GIMP_INT_COMBO_BOX (combo_box),
                               param->store.model_type,
                               G_CALLBACK (gimp_int_combo_box_get_active),
                               &(param->store.model_type));
-  g_signal_connect_swapped (combo_box, "changed",
+  g_signal_connect_swapped (G_OBJECT (combo_box), "changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
   gtk_box_pack_start (GTK_BOX (vbox), combo_box, FALSE, FALSE, 0);
   gtk_widget_show (combo_box);
+  param->widgets[FBLUR_WIDGET_MODEL_TYPE] = combo_box;
 
-  /* the radius to begin to be descending the shine flooding */
+  /* Shining radius */
 
   table = gtk_table_new (1, 2, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 6);
-  //gtk_table_set_row_spacings (GTK_TABLE (table), 6);
   gtk_box_pack_start (GTK_BOX (nb_vbox), table, FALSE, FALSE, 0);
   gtk_widget_show (table);
 
   spinbutton = gimp_spin_button_new (&adj, param->store.shine_radius,
                                      0.0, 100.0, 1.0, 10.0, 0.0, 1.0, 2);
-  g_signal_connect (adj, "value-changed",
+  g_signal_connect (G_OBJECT (adj), "value-changed",
                     G_CALLBACK (gimp_float_adjustment_update),
                     &(param->store.shine_radius));
-  g_signal_connect_swapped (adj, "value-changed",
+  g_signal_connect_swapped (G_OBJECT (adj), "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
   gtk_table_attach (GTK_TABLE (table), spinbutton, 1, 2, 2, 3,
                     GTK_FILL, GTK_FILL, 0, 0);
   gtk_widget_show (spinbutton);
 
-  align = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
-  gtk_table_attach_defaults (GTK_TABLE (table), align, 0, 1, 2, 3);
-  gtk_widget_show (align);
-
   label = gtk_label_new_with_mnemonic (_("P_eak radius for Shining:"));
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), spinbutton);
-  gtk_container_add (GTK_CONTAINER (align), label);
+  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+  gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 2, 3);
   gtk_widget_show (label);
 
   /* Depth Map */
@@ -425,22 +266,21 @@ focusblur_dialog (FblurParam *param)
   toggle = gtk_check_button_new_with_mnemonic (_("_Use Depth map:"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle),
                                 param->store.enable_depth_map);
-  g_signal_connect (toggle, "toggled",
+  g_signal_connect (G_OBJECT (toggle), "toggled",
                     G_CALLBACK (gimp_toggle_button_update),
                     &(param->store.enable_depth_map));
-  g_signal_connect_swapped (toggle, "toggled",
+  g_signal_connect_swapped (G_OBJECT (toggle), "toggled",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);
   gtk_widget_show (toggle);
-  param->widgets[FBLUR_WIDGET_ENABLE_DEPTH_MAP] = toggle;
 
   combo_box = gimp_drawable_combo_box_new (focusblur_map_constraint, NULL);
   gimp_int_combo_box_connect (GIMP_INT_COMBO_BOX (combo_box),
                               param->store.depth_map_ID,
                               G_CALLBACK (gimp_int_combo_box_get_active),
                               &(param->store.depth_map_ID));
-  g_signal_connect_swapped (combo_box, "changed",
+  g_signal_connect_swapped (G_OBJECT (combo_box), "changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
   gtk_widget_set_sensitive (combo_box, param->store.enable_depth_map);
@@ -464,31 +304,78 @@ focusblur_dialog (FblurParam *param)
                               param->store.focal_depth,
                               0.0, 100.0, 1.0, 10.0, 2, TRUE, 0, 0,
                               NULL, NULL);
-  g_signal_connect (adj, "value-changed",
+  g_signal_connect (G_OBJECT (adj), "value-changed",
                     G_CALLBACK (gimp_float_adjustment_update),
                     &(param->store.focal_depth));
-  g_signal_connect_swapped (adj, "value-changed",
+  g_signal_connect_swapped (G_OBJECT (adj), "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
   param->widgets[FBLUR_WIDGET_FOCAL_DEPTH] = GIMP_SCALE_ENTRY_SPINBUTTON (adj);
 
+
   /* Model parameters */
 
   table = focusblur_notebook_append_table (GTK_NOTEBOOK (notebook),
                                            _("_Model"), 3);
   row = 0;
 
-  /* Model fill */
+  /* Brush select button */
+
+  button = gimp_brush_select_button_new (NULL, param->brush_name, -1, -1, -1);
+  gtk_table_attach (GTK_TABLE (table), button, 0, 3, row, row + 1,
+                    GTK_FILL, GTK_FILL, 0, 0);
+  g_signal_connect (G_OBJECT (button), "notify::brush-name",
+                    G_CALLBACK (focusblur_change_brush_name),
+                    &(param->brush_name));
+  g_signal_connect (G_OBJECT (button), "notify::brush-name",
+                    G_CALLBACK (focusblur_update_widget_brushmodel),
+                    param->widgets[FBLUR_WIDGET_MODEL_TYPE]);
+  g_signal_connect_swapped (G_OBJECT (button), "notify::brush-name",
+                            G_CALLBACK (gimp_preview_invalidate),
+                            preview);
+  gtk_widget_show (button);
+  row ++;
+
+  /* Brush balance */
+
+  toggle = gtk_check_button_new_with_mnemonic (_("Make a _balance of brush"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle),
+                                param->store.enable_brush_balance);
+  g_signal_connect (G_OBJECT (toggle), "toggled",
+                    G_CALLBACK (gimp_toggle_button_update),
+                    &(param->store.enable_brush_balance));
+  g_signal_connect_swapped (G_OBJECT (toggle), "toggled",
+                            G_CALLBACK (gimp_preview_invalidate),
+                            preview);
+  gtk_table_attach_defaults (GTK_TABLE (table), toggle, 0, 3, row, row + 1);
+  row ++;
+  gtk_widget_show (toggle);
+
+  /* Model rotation */
+
+  adj = gimp_scale_entry_new (GTK_TABLE (table), 0, row ++,
+                              _("_Rotation:"), 100, 0,
+                              param->store.model_rotate,
+                              0.0, 360.0, 1.0, 10.0, 2, TRUE, 0, 0,
+                              NULL, NULL);
+  g_signal_connect (G_OBJECT (adj), "value-changed",
+                    G_CALLBACK (gimp_float_adjustment_update),
+                    &(param->store.model_rotate));
+  g_signal_connect_swapped (G_OBJECT (adj), "value-changed",
+                            G_CALLBACK (gimp_preview_invalidate),
+                            preview);
+
+  /* Model filling */
 
   adj = gimp_scale_entry_new (GTK_TABLE (table), 0, row ++,
                               _("F_illing inside:"), 100, 0,
                               param->store.model_fill,
                               0.0, 100.0, 1.0, 10.0, 2, TRUE, 0, 0,
                               NULL, NULL);
-  g_signal_connect (adj, "value-changed",
+  g_signal_connect (G_OBJECT (adj), "value-changed",
                     G_CALLBACK (gimp_float_adjustment_update),
                     &(param->store.model_fill));
-  g_signal_connect_swapped (adj, "value-changed",
+  g_signal_connect_swapped (G_OBJECT (adj), "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
 
@@ -499,26 +386,13 @@ focusblur_dialog (FblurParam *param)
                               param->store.model_softness,
                               0.0, 100.0, 1.0, 10.0, 2, TRUE, 0, 0,
                               NULL, NULL);
-  g_signal_connect (adj, "value-changed",
+  g_signal_connect (G_OBJECT (adj), "value-changed",
                     G_CALLBACK (gimp_float_adjustment_update),
                     &(param->store.model_softness));
-  g_signal_connect_swapped (adj, "value-changed",
+  g_signal_connect_swapped (G_OBJECT (adj), "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
 
-  /* Model softness delay */
-
-  adj = gimp_scale_entry_new (GTK_TABLE (table), 0, row ++,
-                              _("Soft_ness delay:"), 100, 0,
-                              param->store.model_softness_delay,
-                              0.0, 100.0, 1.0, 10.0, 2, TRUE, 0, 0,
-                              NULL, NULL);
-  g_signal_connect (adj, "value-changed",
-                    G_CALLBACK (gimp_float_adjustment_update),
-                    &(param->store.model_softness_delay));
-  g_signal_connect_swapped (adj, "value-changed",
-                            G_CALLBACK (gimp_preview_invalidate),
-                            preview);
 
   /* Shine parameters */
 
@@ -528,14 +402,12 @@ focusblur_dialog (FblurParam *param)
 
   /* Shine type */
 
-  combo_box = gimp_int_combo_box_new (_("Luminosity"),  FBLUR_SHINE_LUMINOSITY,
-                                      _("Saturation"),  FBLUR_SHINE_SATURATION,
-                                      NULL);
+  combo_box = gimp_enum_combo_box_new (FBLUR_TYPE_SHINE_TYPE);
   gimp_int_combo_box_connect (GIMP_INT_COMBO_BOX (combo_box),
                               param->store.shine_type,
                               G_CALLBACK (gimp_int_combo_box_get_active),
                               &(param->store.shine_type));
-  g_signal_connect_swapped (combo_box, "changed",
+  g_signal_connect_swapped (G_OBJECT (combo_box), "changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
   label = gimp_table_attach_aligned (GTK_TABLE (table), 0, row ++,
@@ -550,28 +422,28 @@ focusblur_dialog (FblurParam *param)
                               param->store.shine_threshold,
                               0.0, 100.0, 1.0, 10.0, 2, TRUE, 0, 0,
                               NULL, NULL);
-  g_signal_connect (adj, "value-changed",
+  g_signal_connect (G_OBJECT (adj), "value-changed",
                     G_CALLBACK (gimp_float_adjustment_update),
                     &(param->store.shine_threshold));
-  g_signal_connect_swapped (adj, "value-changed",
+  g_signal_connect_swapped (G_OBJECT (adj), "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
 
-  /* Shine Level */
+  /* Shine level */
 
   adj = gimp_scale_entry_new (GTK_TABLE (table), 0, row ++,
                               _("_Level:"), 100, 0,
                               param->store.shine_level,
                               0.0, 100.0, 1.0, 10.0, 2, TRUE, 0, 0,
                               NULL, NULL);
-  g_signal_connect (adj, "value-changed",
+  g_signal_connect (G_OBJECT (adj), "value-changed",
                     G_CALLBACK (gimp_float_adjustment_update),
                     &(param->store.shine_level));
-  g_signal_connect_swapped (adj, "value-changed",
+  g_signal_connect_swapped (G_OBJECT (adj), "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
 
-  /* Shine Curve */
+  /* Shine curve */
 
   adj = gimp_scale_entry_new (GTK_TABLE (table), 0, row ++,
                               _("C_urve:"), 100, 0,
@@ -579,13 +451,14 @@ focusblur_dialog (FblurParam *param)
                               0.0, 10.0, 0.1, 1.0, 3, TRUE, 0, 0,
                               NULL, NULL);
   gimp_scale_entry_set_logarithmic (adj, TRUE);
-  g_signal_connect (adj, "value-changed",
+  g_signal_connect (G_OBJECT (adj), "value-changed",
                     G_CALLBACK (gimp_float_adjustment_update),
                     &(param->store.shine_curve));
-  g_signal_connect_swapped (adj, "value-changed",
+  g_signal_connect_swapped (G_OBJECT (adj), "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
 
+
   /* Depth parameters */
 
   table = focusblur_notebook_append_table (GTK_NOTEBOOK (notebook),
@@ -597,29 +470,34 @@ focusblur_dialog (FblurParam *param)
   toggle = gtk_check_button_new_with_mnemonic (_("Depth with p_recedence"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle),
                                 param->store.enable_depth_precedence);
-  g_signal_connect (toggle, "toggled",
+  g_signal_connect (G_OBJECT (toggle), "toggled",
                     G_CALLBACK (gimp_toggle_button_update),
                     &(param->store.enable_depth_precedence));
-  g_signal_connect_swapped (toggle, "toggled",
+  g_signal_connect_swapped (G_OBJECT (toggle), "toggled",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
   gtk_table_attach_defaults (GTK_TABLE (table), toggle, 0, 3, row, row + 1);
   row ++;
   gtk_widget_show (toggle);
 
-#ifdef HAVE_FFTW3
 
-  /* Quality parameters */
+#if defined HAVE_FFTW3 || defined ENABLE_MP
+
+  /* Recordable parameters */
 
   table = focusblur_notebook_append_table (GTK_NOTEBOOK (notebook),
                                            _("Pre_ferences"), 2);
   row = 0;
 
+#  ifdef HAVE_FFTW3
+
   /* Rendering quality */
-  combo_box = gimp_int_combo_box_new (_("Best"),        FBLUR_QUALITY_BEST,
-                                      _("Normal"),      FBLUR_QUALITY_NORMAL,
-                                      _("Low"),         FBLUR_QUALITY_LOW,
-                                      NULL);
+
+  enum_store = gimp_enum_store_new_with_range
+    (FBLUR_TYPE_QUALITY_TYPE, FBLUR_QUALITY_BEST, FBLUR_QUALITY_LOW);
+  combo_box =
+    gimp_enum_combo_box_new_with_model (GIMP_ENUM_STORE (enum_store));
+  g_object_unref (enum_store);
   gimp_int_combo_box_connect (GIMP_INT_COMBO_BOX (combo_box),
                               param->pref.quality,
                               G_CALLBACK (gimp_int_combo_box_get_active),
@@ -629,11 +507,13 @@ focusblur_dialog (FblurParam *param)
                                      combo_box, 2, FALSE);
   gtk_widget_show (combo_box);
 
-  /* Detail: preview quality */
-  combo_box = gimp_int_combo_box_new (_("Normal"),      FBLUR_QUALITY_NORMAL,
-                                      _("Low"),         FBLUR_QUALITY_LOW,
-                                      _("Defective"), FBLUR_QUALITY_DEFECTIVE,
-                                      NULL);
+  /* Preview quality */
+
+  enum_store = gimp_enum_store_new_with_range
+    (FBLUR_TYPE_QUALITY_TYPE, FBLUR_QUALITY_NORMAL, FBLUR_QUALITY_DEFECTIVE);
+  combo_box =
+    gimp_enum_combo_box_new_with_model (GIMP_ENUM_STORE (enum_store));
+  g_object_unref (enum_store);
   gimp_int_combo_box_connect (GIMP_INT_COMBO_BOX (combo_box),
                               param->pref.quality_preview,
                               G_CALLBACK (gimp_int_combo_box_get_active),
@@ -643,9 +523,9 @@ focusblur_dialog (FblurParam *param)
                                      combo_box, 2, FALSE);
   gtk_widget_show (combo_box);
 
-#endif /* HAVE_FFTW3 */
+#  endif /* HAVE_FFTW3 */
 
-#ifdef ENABLE_MP
+#  ifdef ENABLE_MP
 
   /* Disable multi-threads */
 
@@ -662,7 +542,9 @@ focusblur_dialog (FblurParam *param)
   row ++;
   gtk_widget_show (toggle);
 
-#endif /* ENABLE_MP */
+#  endif /* ENABLE_MP */
+
+#endif /* defined HAVE_FFTW3 || defined ENABLE_MP */
 
 
   /* get parameters */
@@ -675,34 +557,6 @@ focusblur_dialog (FblurParam *param)
 }
 
 
-void
-focusblur_widget_set_enable_depth_map (FblurParam       *param,
-                                       gboolean          bool)
-{
-  GtkWidget *widget;
-
-  widget = param->widgets[FBLUR_WIDGET_ENABLE_DEPTH_MAP];
-  if (widget)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), bool);
-  else
-    param->store.enable_depth_map = bool;
-}
-
-
-void
-focusblur_widget_set_enable_shine (FblurParam   *param,
-                                   gboolean      bool)
-{
-  GtkWidget *widget;
-
-  widget = param->widgets[FBLUR_WIDGET_ENABLE_SHINE];
-  if (widget)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), bool);
-  else
-    param->store.enable_shine = bool;
-}
-
-
 static void
 focusblur_preview_invalidate (GimpPreview *gimppreview,
                               FblurParam  *param)
@@ -722,9 +576,17 @@ focusblur_widget_pickup_focal_depth (FblurParam *param,
   guchar                *pixel;
   gint                   num_channels;
 
-  if (! param->store.enable_depth_map ||
-      ! gimp_drawable_is_valid (param->store.depth_map_ID))
-    return;
+  g_return_if_fail (param->store.enable_depth_map);
+  g_return_if_fail (x >= 0 && y >= 0);
+
+  if (! gimp_drawable_is_valid (param->store.depth_map_ID))
+    {
+      gimp_message (_("Specified depth map is invalid."));
+      return;
+    }
+
+  x %= gimp_drawable_width (param->store.depth_map_ID);
+  y %= gimp_drawable_height (param->store.depth_map_ID);
 
   pixel = gimp_drawable_get_pixel
     (param->store.depth_map_ID, x, y, &num_channels);
@@ -769,21 +631,36 @@ focusblur_preview_button_press (GtkWidget       *widget,
                                 GdkEvent        *event,
                                 FblurParam      *param)
 {
-  GimpPreviewArea       *area;
-  gint                   x, y;
+  if (event->button.button == 2)
+    {
+      GimpPreviewArea *area;
+      gint x, y;
 
-  if (event->button.button != 2)
-    return;
+      if (! param->store.enable_depth_map ||
+          ! gimp_drawable_is_valid (param->store.depth_map_ID))
+        return;
 
-  if (! param->store.enable_depth_map ||
-      ! gimp_drawable_is_valid (param->store.depth_map_ID))
-    return;
+      area = GIMP_PREVIEW_AREA (widget);
+      x = area->offset_x + event->button.x;
+      y = area->offset_y + event->button.y;
 
-  area = GIMP_PREVIEW_AREA (widget);
-  x = area->offset_x + event->button.x;
-  y = area->offset_y + event->button.y;
+      /* checks for padding */
+      if (area->width < widget->allocation.width)
+        {
+          x -= (widget->allocation.width - area->width) / 2;
+          if (x < 0 || x >= area->max_width)
+            return;
+        }
+      if (area->height < widget->allocation.height)
+        {
+          y -= (widget->allocation.height - area->height) / 2;
+          if (y < 0 || y >= area->max_height)
+            return;
+        }
 
-  focusblur_widget_pickup_focal_depth (param, x, y);
+      focusblur_widget_pickup_focal_depth (param, x, y);
+      return;
+    }
 }
 
 
@@ -798,3 +675,65 @@ focusblur_map_constraint (gint32         image_ID,
   return (gimp_drawable_is_rgb (drawable_ID) ||
           gimp_drawable_is_gray (drawable_ID));
 }
+
+
+static void
+focusblur_change_brush_name (GObject    *object,
+                             GParamSpec *arg1,
+                             gpointer   user_data)
+{
+  gchar **fblur_brush_name = (gchar **) user_data;
+  gchar *gimp_brush_name = NULL;
+
+  g_return_if_fail (GIMP_IS_BRUSH_SELECT_BUTTON (object));
+  g_return_if_fail (fblur_brush_name != NULL);
+
+  g_object_get (object, "brush-name", &gimp_brush_name, NULL);
+
+  if (gimp_brush_name)
+    {
+      g_free (*fblur_brush_name);
+      *fblur_brush_name = g_strdup (gimp_brush_name);
+    }
+}
+
+static void
+focusblur_update_enumcombobox_text (GimpEnumComboBox *combo_box,
+                                    gint              index,
+                                    const gchar      *text)
+{
+  GimpEnumStore         *model;
+  GtkTreeIter            iter;
+  gboolean               ret;
+
+  g_object_get (GIMP_ENUM_COMBO_BOX (combo_box), "model", &model, NULL);
+
+  ret = gimp_int_store_lookup_by_value (GTK_TREE_MODEL (model), index, &iter);
+
+  if (ret)
+    gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+                        GIMP_INT_STORE_LABEL, text,
+                        -1);
+}
+
+
+static void
+focusblur_update_widget_brushmodel (GObject    *object,
+                                    GParamSpec *arg1,
+                                    gpointer    user_data)
+{
+  gchar *gimp_brush_name = NULL;
+  gchar *text;
+
+  g_return_if_fail (GIMP_IS_BRUSH_SELECT_BUTTON (object));
+  g_return_if_fail (GIMP_IS_ENUM_COMBO_BOX (user_data));
+
+  g_object_get (object, "brush-name", &gimp_brush_name, NULL);
+
+  focusblur_stock_update_modelbrush (gimp_brush_name);
+
+  text = g_strdup_printf (_("Brush:%s"), gimp_brush_name);
+  focusblur_update_enumcombobox_text (GIMP_ENUM_COMBO_BOX (user_data),
+                                      FBLUR_MODEL_BRUSH, text);
+  g_free (text);
+}
