@@ -22,18 +22,24 @@
 #ifndef __FOCUSBLUR_RENDER_H__
 #define __FOCUSBLUR_RENDER_H__
 
+#include "config.h"
+
 #include <glib/gtypes.h>
+//#include <libgimp/gimp.h>
+#include <libgimp/gimpui.h>
 
 #include "focusblurtypes.h"
 
 
 G_BEGIN_DECLS
 
-void    focusblur_render_pixel          (gint            x,
-                                         gint            y,
-                                         guchar         *dest,
-                                         gint            bpp,
-                                         FblurParam     *param);
+
+gboolean        focusblur_execute               (FblurParam     *param,
+                                                 GimpPreview    *preview);
+#ifdef ENABLE_MP
+void            focusblur_execute_thread        (gpointer        data,
+                                                 gpointer        user_data);
+#endif
 
 
 G_END_DECLS
