@@ -1,6 +1,5 @@
 /* GIMP LiquidRescaling Plug-in
  * Copyright (C) 2007 Carlo Baldassi (the "Author") <carlobaldassi@yahoo.it>.
- * (implementation based on the GIMP Plug-in Template by Michael Natterer)
  * All Rights Reserved.
  *
  * This plugin implements the algorithm described in the paper
@@ -10,7 +9,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 dated June, 1991.
+ * the Free Software Foundation; either version 2 of the Licence, or
+ * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,14 +18,15 @@
  * GNU General Public License for more details.
  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org.licences/>.
  */
 
 #ifndef __RENDER_H__
 #define __RENDER_H__
 
+/* progress functions wrappers */
+gboolean my_progress_end (const gchar * message);
+
 /*  Render function  */
 
 gboolean render (gint32 image_ID,
